@@ -19,4 +19,10 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 #define realloc(p,s) myrealloc(p,s)
 #endif
 
+#ifndef _POSIX_PATH_MAX
+#ifdef MAX_PATH
+#define _POSIX_PATH_MAX MAX_PATH
+#endif
+#endif
+
 #endif
