@@ -1,12 +1,12 @@
 /* Notes */ /*{{{C}}}*//*{{{*/
 /*
 
-This program is GNU software, copyright 1997, 1998, 1999, 2000, 2001
+This program is GNU software, copyright 1997-2007
 Michael Haardt <michael@moria.de>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 This program is distributed in the hope that it will be useful, but
@@ -21,12 +21,12 @@ with this program.  If not, write to the Free Software Foundation, Inc.,
 */
 /*}}}*/
 /* #includes */ /*{{{*/
-#ifndef NO_POSIX_SOURCE
 #undef  _POSIX_SOURCE
 #define _POSIX_SOURCE   1
 #undef  _POSIX_C_SOURCE
 #define _POSIX_C_SOURCE 2
-#endif
+
+#include "config.h"
 
 #include <sys/types.h>
 #include <assert.h>
@@ -34,13 +34,11 @@ with this program.  If not, write to the Free Software Foundation, Inc.,
 #include <errno.h>
 #include <limits.h>
 #include <locale.h>
-#ifdef HAVE_NL_TYPES_H
-#include <nl_types.h>
+#ifdef HAVE_GETTEXT
+#include <libintl.h>
+#define _(String) gettext(String)
 #else
-typedef long nl_catd;
-static nl_catd catopen(const char *name, int oflag) { return 0; }
-static const char *catgets(nl_catd catd, int set_id, int msg_id, const char *msg) { return msg; }
-static void catclose(nl_catd catd) { }
+#define _(String) String
 #endif
 #include <regex.h>
 #include <stdio.h>
@@ -50,10 +48,6 @@ static void catclose(nl_catd catd) { }
 #include "misc.h"
 #include "sentence.h"
 /*}}}*/
-/* #defines */ /*{{{*/
-#define OOPS  catgets(catd,2,1,"%s: internal error, can not compile regular expression: %s\n")
-#define NOMEM catgets(catd,2,2,"%s: increasing sentence buffer failed: %s\n")
-/*}}}*/
 
 static const char *abbreviations_de[]= /*{{{*/
 {
@@ -118,6 +112,30 @@ static const char *abbreviations_en[]= /*{{{*/
   (const char*)0
 };
 /*}}}*/
+static const char *abbreviations_nl[]= /*{{{*/
+{
+  "as",
+  "aub",
+  "bijv",
+  "bv",
+  "ca",
+  "dd",
+  "dr",
+  "drs",
+  "ed",
+  "jl",
+  "maw",
+  "muv",
+  "oa",
+  "tav",
+  "tzt",
+  "zg",
+  "zgn",
+  "zoz",
+  "zsm",
+  (const char*)0
+};
+/*}}}*/
 static const char *abbreviations_none[]= /*{{{*/
 {
   (const char*)0
@@ -125,12 +143,13 @@ static const char *abbreviations_none[]= /*{{{*/
 /*}}}*/
 static const char **abbreviations;
 
-static int endingInAbbrev(const char *s, size_t length) /*{{{*/
+static int endingInAbbrev(const char *s, size_t length, const char *lang) /*{{{*/
 {
   const char **abbrev=abbreviations;
   size_t aLength;
 
   if (!isalpha(s[length-1])) return 0;
+  if (endingInPossesiveS(s,length)) return 0;
   while (*abbrev!=(const char*)0)
   {
     if ((aLength=strlen(*abbrev))<length)
@@ -149,14 +168,21 @@ static int endingInAbbrev(const char *s, size_t length) /*{{{*/
 }
 /*}}}*/
 
-void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*process)(const char *, size_t, const char *, int), const char *lang) /*{{{*/
+int endingInPossesiveS(const char *s, size_t length) /*{{{*/
+{
+  return (abbreviations==abbreviations_en && length>=3 && strncmp(s+length-2,"\'s",2)==0);
+}
+/*}}}*/
+void sentence(const char *cmd, FILE *in, const char *file, void (*process)(const char *, size_t, const char *, int), const char *lang) /*{{{*/
 {
   /* variables */ /*{{{*/
   int voc,oc,c;
   char *sent=malloc(128);
   size_t length=0,capacity=128;
+  int inSentence=0;
   int inWhiteSpace=0;
   int inParagraph=0;
+  int ellipsis=0;
   int line=1,beginLine=1;
   int err;
   regex_t hashLine;
@@ -166,6 +192,7 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
   if (strncmp(lang,"en",2)==0) abbreviations=abbreviations_en;
   else if (strncmp(lang,"C",1)==0) abbreviations=abbreviations_en;
   else if (strncmp(lang,"de",2)==0) abbreviations=abbreviations_de;
+  else if (strncmp(lang,"nl",2)==0) abbreviations=abbreviations_nl;		/* JDL */
   else abbreviations=abbreviations_none;
   /* compile #line number "file" regular expression */ /*{{{*/
   if ((err=regcomp(&hashLine,"^[ \t]*line[ \t]*\\([0-9][0-9]*\\)[ \t]*\"\\([^\"]*\\)\"",0)))
@@ -173,7 +200,7 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
     char buf[256];
     size_t len=regerror(err,&hashLine,buf,sizeof(buf)-1);
     buf[len]='\0';
-    fprintf(stderr,OOPS,cmd,buf);
+    fprintf(stderr,_("%s: internal error, compiling a regular expression failed (%s).\n"),cmd,buf);
     exit(2);
   }
   /*}}}*/
@@ -194,7 +221,7 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
       {
         size_t len;
 
-        line=strtol(buf+found[1].rm_so,(char**)0,10);
+        line=strtol(buf+found[1].rm_so,(char**)0,10)-1;
         len=found[2].rm_eo-found[2].rm_so;
         if (len>_POSIX_PATH_MAX) len=_POSIX_PATH_MAX;
         strncpy(filebuf,buf+found[2].rm_so,len);
@@ -202,7 +229,7 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
         file=filebuf;
       }
       /*}}}*/
-      c=getc(in);
+      c='\n';
       continue;
     }
     /*}}}*/
@@ -210,7 +237,7 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
     {
       if (length>=(capacity-1) && (sent=realloc(sent,capacity*=2))==(char*)0)
       {
-        fprintf(stderr,NOMEM,cmd,strerror(errno));
+        fprintf(stderr,_("%s: increasing sentence buffer failed: %s\n"),cmd,strerror(errno));
         exit(2);
       }
       if (isspace(oc))
@@ -221,18 +248,60 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
           inWhiteSpace=1;
         }
       }
-      else if ((oc=='.' || oc==':' || oc=='!' || oc=='?') && (c==EOF || isspace(c)) && (oc!='.' || !endingInAbbrev(sent,length)))
-      {
-        if (inWhiteSpace) --length;
-        sent[length++]=oc;
-        sent[length]='\0';
-        process(sent,length,file,beginLine);
-        length=0;
-      }
       else
       {
         sent[length++]=oc;
-        inWhiteSpace=0;
+        if (isalpha(oc)) inSentence=1;
+        if
+        (
+          (length==3 && strncmp(sent+length-3,"...",3)==0 && (c==EOF || isspace(c)))
+          || (length>=4 && strncmp(sent+length-4," ...",4)==0 && (c==EOF || isspace(c)))
+        )
+        {
+          /* omission ellipsis */
+          inWhiteSpace=0;
+        }
+        else if (length>=4 && !isspace(sent[length-4]) && strncmp(sent+length-3,"...",3)==0 && (c==EOF || isspace(c)))
+        {
+          /* beginning ellipsis */
+          char foo;
+
+          foo=sent[length-3];
+          sent[length-3]='\0';
+          if (inSentence) process(sent,length-3,file,beginLine);
+          sent[length-3]=foo;
+          memmove(sent,sent+length-3,3);
+          length=3;
+          inParagraph=0;
+          inWhiteSpace=0;
+          beginLine=line;
+          inSentence=0;
+        }
+        else if (length>=4 && strncmp(sent+length-4,"...",3)==0 && (c==EOF || isspace(c)))
+        {
+          /* ending ellipsis */
+          if (inWhiteSpace) --length;
+          sent[length]='\0';
+          if (inSentence) process(sent,length,file,beginLine);
+          length=0;
+          inWhiteSpace=0;
+          inSentence=0;
+        }
+        else if ((oc=='.' || oc==':' || oc=='!' || oc=='?') && (c==EOF || isspace(c) || c=='"') && (!isdigit(voc) || oc!='.' || !isdigit(c)) && (oc!='.' || !endingInAbbrev(sent,length,lang)))
+        {
+          /* end of sentence */
+          if (inWhiteSpace) --length;
+          sent[length]='\0';
+          if (inSentence) process(sent,length,file,beginLine);
+          length=0;
+          inWhiteSpace=0;
+          inSentence=0;
+        }
+        else
+        {
+          /* just a regular character */
+          inWhiteSpace=0;
+        }
       }
     }
     else if (isupper(oc))
@@ -241,8 +310,9 @@ void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*
       sent[length++]=oc;
       inWhiteSpace=0;
       beginLine=line;
+      inSentence=1;
     }
-    else if (!inParagraph && oc=='\n' && c==oc)
+    else if (!inParagraph && oc=='\n' && c=='\n')
     {
       process("",0,file,line);
       inParagraph=1;
