@@ -1,12 +1,12 @@
 /* Notes */ /*{{{C}}}*//*{{{*/
 /*
 
-This program is GNU software, copyright 1997, 1998, 1999, 2000, 2001
+This program is GNU software, copyright 1997-2007
 Michael Haardt <michael@moria.de>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 This program is distributed in the hope that it will be useful, but
@@ -27,6 +27,7 @@ with this program.  If not, write to the Free Software Foundation, Inc.,
 #include <sys/types.h>
 #include <stdio.h>
 
-void sentence(const char *cmd, nl_catd catd, FILE *in, const char *file, void (*process)(const char *, size_t, const char *, int), const char *lang);
+int endingInPossesiveS(const char *s, size_t length);
+void sentence(const char *cmd, FILE *in, const char *file, void (*process)(const char *, size_t, const char *, int), const char *lang);
 
 #endif
