@@ -1,12 +1,12 @@
 /* Notes */ /*{{{C}}}*//*{{{*/
 /*
 
-This program is GNU software, copyright 1997, 1998, 1999, 2000, 2001
+This program is GNU software, copyright 1997-2007
 Michael Haardt <michael@moria.de>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 This program is distributed in the hope that it will be useful, but
@@ -20,74 +20,36 @@ with this program.  If not, write to the Free Software Foundation, Inc.,
 
 */
 /*}}}*/
+
 /* #includes */ /*{{{*/
 #undef  _POSIX_SOURCE
 #define _POSIX_SOURCE   1
 #undef  _POSIX_C_SOURCE
 #define _POSIX_C_SOURCE 2
 
+#include "config.h"
+
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 #include <locale.h>
-#include <math.h>
-#ifdef HAVE_NL_TYPES_H
-#include <nl_types.h>
+#ifdef HAVE_GETTEXT
+#include <libintl.h>
+#define _(String) gettext(String)
 #else
-typedef long nl_catd;
-static nl_catd catopen(const char *name, int oflag) { return 0; }
-static const char *catgets(nl_catd catd, int set_id, int msg_id, const char *msg) { return msg; }
-static void catclose(nl_catd catd) { }
+#define _(String) String
 #endif
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "getopt.h"
-#include "config.h"
-
 #include "misc.h"
 #include "sentence.h"
 /*}}}*/
-/* #defines */ /*{{{*/
-#define USAGE1     catgets(catd,1, 1,"Usage: style [-L language] [-l length] [-r ari] [file ...]\n")
-#define USAGE2     catgets(catd,1, 2,"       style [--language language] [--print-long length] [--print-ari ari]\n             [file ...]\n")
-#define USAGE3     catgets(catd,1, 3,"       style --version\n")
-#define MORE       catgets(catd,1, 4,"Try style -h|--help for more information.\n")
-#define HELP1      catgets(catd,1, 5,"Analyse surface characteristics of a document.\n")
-#define HELP2      catgets(catd,1, 6,"-L, --language          set the document language.\n")
-#define HELP3      catgets(catd,1, 7,"-l, --print-long        print all sentences longer than <length> words\n")
-#define HELP4      catgets(catd,1, 8,"-r, --print-ari         print all sentences with an ARI greater than than <ari>\n")
-#define HELP5      catgets(catd,1, 9,"-p, --print-passive     print all sentences phrased in the passive voice\n")
-#define HELP6      catgets(catd,1,10,"-N, --print-nom         print all sentences containing nominalizations\n")
-#define HELP7      catgets(catd,1,11,"-n, --print-nom-passive print all sentences phrased in the passive voice or\n                        containing nominalizations\n")
-#define HELP8      catgets(catd,1,12,"-h, --help              print this message\n")
-#define HELP9      catgets(catd,1,13,"    --version           print the version\n")
-#define BUGS       catgets(catd,1,14,"Report bugs to <michael@moria.de>.\n")
-#define OPENFAILED catgets(catd,1,15,"style: can not open %s: %s\n")
-#define NOSENT     catgets(catd,1,16,"No sentences found.\n")
-#define NOMEM      catgets(catd,1,17,"style: out of memory\n")
-#define GRADES     catgets(catd,1,18,"readability grades:\n")
-#define KINCAID    catgets(catd,1,19,"Kincaid")
-#define ARI        catgets(catd,1,20,"ARI")
-#define CL         catgets(catd,1,21,"Coleman-Liau")
-#define FLESCH     catgets(catd,1,22,"Flesch Index")
-#define FOG        catgets(catd,1,23,"Fog Index")
-#define WSFT       catgets(catd,1,24,"1. WSTF Index")
-#define WSI        catgets(catd,1,25,"Wheeler-Smith Index")
-#define BELOW5     catgets(catd,1,26,"below school year 5\n")
-#define AHEAD10    catgets(catd,1,27,"higher than school year 10\n")
-#define YEAR       catgets(catd,1,28,"school year %d\n")
-#define LIX        catgets(catd,1,29,"Lix")
-#define AHEAD11    catgets(catd,1,30,"higher than school year 11\n")
-#define SMOG       catgets(catd,1,31,"SMOG-Grading")
-#define SENTINFO   catgets(catd,1,32,"sentence info:\n")
-#define CHARS      catgets(catd,1,33,"        %d characters\n")
-#define SHORTSENT  catgets(catd,1,34,"        %d%% (%d) short sentences (at most %d words)\n")
-/*}}}*/
 
 /* variables */ /*{{{*/
-static nl_catd catd;
 static const char *lc_ctype;
 enum lc_ctype_int { ASCII, ISO_8859_1 };
 static enum lc_ctype_int lc_ctype_int;
@@ -107,7 +69,7 @@ static void newHit(struct Hit *hit) /*{{{*/
 {
   if ((hit->data=malloc((hit->capacity=3)*sizeof(int)))==(int*)0)
   {
-    fprintf(stderr,NOMEM);
+    fprintf(stderr,_("style: out of memory\n"));
     exit(1);
   }
   memset(hit->data,0,hit->capacity*sizeof(int));
@@ -121,7 +83,7 @@ static void noteHit(struct Hit *hit, int n) /*{{{*/
   {
     if ((hit->data=realloc(hit->data,n*2*sizeof(int)))==(int*)0)
     {
-      fprintf(stderr,NOMEM);
+      fprintf(stderr,_("style: out of memory\n"));
       exit(1);
     }
     memset(hit->data+hit->capacity,0,(n*2-hit->capacity)*sizeof(int));
@@ -165,7 +127,12 @@ static double ari(int letters, int words, int sentences) /*{{{*/
  */
 static double coleman_liau(int letters, int words, int sentences) /*{{{*/
 {
+#if 0
+  /* where did I get this from? */
   return 5.89*(((double)letters)/words)-0.3*(((double)sentences)/(100*words))-15.8;
+#else
+  return 5.879851*letters/words-29.587280*sentences/words-15.800804;
+#endif
 }
 /*}}}*/
 
@@ -218,16 +185,17 @@ static double wstf(int words, int shortwords, int longwords, int bigwords, int s
  */
 static double wheeler_smith(int *grade, int words, int bigwords, int sentences) /*{{{*/
 {
-  double index=(((double)words)/sentences) * 10.0 * (((double)bigwords)/words);
-  if (index<=16) *grade=0;
-  else if (index<=20) *grade=5;
-  else if (index<=24) *grade=6;
-  else if (index<=29) *grade=7;
-  else if (index<=34) *grade=8;
-  else if (index<=38) *grade=9;
-  else if (index<=42) *grade=10;
+  double idx=(((double)words)/sentences) * 10.0 * (((double)bigwords)/words);
+
+  if (idx<=16) *grade=0;
+  else if (idx<=20) *grade=5;
+  else if (idx<=24) *grade=6;
+  else if (idx<=29) *grade=7;
+  else if (idx<=34) *grade=8;
+  else if (idx<=38) *grade=9;
+  else if (idx<=42) *grade=10;
   else *grade=99;
-  return index;
+  return idx;
 }
 /*}}}*/
 
@@ -242,17 +210,18 @@ static double wheeler_smith(int *grade, int words, int bigwords, int sentences)
  */
 static double lix(int *grade, int words, int longwords, int sentences) /*{{{*/
 {
-  double index=((double)words)/sentences+100.0*((double)longwords)/words;
-  if (index<34) *grade=0;
-  else if (index<38) *grade=5;
-  else if (index<41) *grade=6;
-  else if (index<44) *grade=7;
-  else if (index<48) *grade=8;
-  else if (index<51) *grade=9;
-  else if (index<54) *grade=10;
-  else if (index<57) *grade=11;
+  double idx=((double)words)/sentences+100.0*((double)longwords)/words;
+
+  if (idx<34) *grade=0;
+  else if (idx<38) *grade=5;
+  else if (idx<41) *grade=6;
+  else if (idx<44) *grade=7;
+  else if (idx<48) *grade=8;
+  else if (idx<51) *grade=9;
+  else if (idx<54) *grade=10;
+  else if (idx<57) *grade=11;
   else *grade=99;
-  return index;
+  return idx;
 }
 /*}}}*/
 
@@ -299,10 +268,18 @@ static int article(const char *word, size_t l) /*{{{*/
     "the", "a", "an", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch articles (lidwoord) */ /*{{{*/
+  {
+    "de", "het", "een", (const char*)0
+  };
+  /*}}}*/
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
@@ -316,9 +293,16 @@ static int pronoun(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Pronomen */ /*{{{*/
   {
-    "ich", "du", "er", "sie", "es", "wir", "ihr", "mein", "meine", "dein",
-    "deine", "sein", "seine", "unser", "unsere", "euer", "eure", "mir",
-    "mich", "dir", "dich", "ihre", (const char*)0
+    /* Nominativ */ "ich", "du", "er", "sie", "es", "wir", "ihr", /* sie */
+    /* Akkusativ */ "mich", "dich", "ihn", "uns", "euch", /* sie */
+    /* Dativ     */ "mir", "dir", "ihm", /* uns euch ihr */ "ihnen",
+    /* Genitiv   */ "mein", "dein", "sein", "unser", "euer", /* ihr */
+    /* Genitiv   */ "meiner", "deiner", "seiner", "unserer", "eurer", "ihrer",
+    /* Genitiv   */ "meine", "deine", "seine", "unsere", "eure", "ihre",
+    /* Genitiv   */ "meines", "deines", "seines", "unseres", "eures", "ihres",
+    /* Genitiv   */ "meinem", "deinem", "seinem", "unserem", "eurem", "ihrem",
+    /* Genitiv   */ "meinen", "deinen", "seinen", "unseren", "euren", "ihren",
+    (const char*)0
   };
   /*}}}*/
   static const char *en[]= /* pronouns */ /*{{{*/
@@ -330,10 +314,26 @@ static int pronoun(const char *word, size_t l) /*{{{*/
     "our", "that", "their", "these", "this", "those", "your", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch pronouns (voornaamwoord) */ /*{{{*/
+  {
+    "ik", 		 	"jij", "je", "u", "gij", "ge", 	"hij", "zij", "ze", "het",			/* persoonlijk voornaamwoord */
+	"wij", "we", 	"jullie", 						/* "zij", "ze", */
+    "me", "mijzelf", "mezelf", "je", "jezelf", "uzelf", 					/* wederkeren voornaamwoord */
+	"zich", "zichzelf", "haarzelf", "onszelf", /* "jezelf", */
+	"elkaar", "elkaars", "elkander", "elkanders", "mekaar", "mekaars",		/* wedekerig voornamwoord */
+    "mijnen",  "deinen", "zijnen",   "haren",    "onzen",   "uwen", "hunnen", "haren",	/* pers. vnw: obsolete naamvallen */
+    "mijner",  "deiner", "zijner",   "harer",    "onzer",   "uwer", "hunner", "harer",
+    "mijnes",  "deines", "zijnes",   "hares",    "onzes",   "uwes", "hunnes", "hares", (const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
@@ -343,6 +343,7 @@ static int pronoun(const char *word, size_t l) /*{{{*/
  * Test if the word is an interrogative pronoun.  This function uses
  * docLanguage to determine the used language.
  */
+ 
 static int interrogativePronoun(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Interrogativpronomen */ /*{{{*/
@@ -356,15 +357,30 @@ static int interrogativePronoun(const char *word, size_t l) /*{{{*/
     "why", "who", "what", "whom", "when", "where", "how", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch interrogative pronouns (vragend voornaamwoord) */ /*{{{*/
+  {
+    "welke", "wat", "wat voor", "wat voor een", "welk", 
+    "wie", "waar", "wanneer", "hoe", (const char*)0
+  };
+  /*}}}*/  
+  
   const char **list;
-
+ 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
 /*}}}*/
 
+/**
+ * Test if the word is an conjunction.  This function uses
+ * docLanguage to determine the used language.
+ */
+ 
 static int conjunction(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Konjunktionen */ /*{{{*/
@@ -378,15 +394,29 @@ static int conjunction(const char *word, size_t l) /*{{{*/
     "and", "but", "or", "yet", "nor", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch conjunctions (nevenschikkend voegwoord) */ /*{{{*/
+  {
+    "en", "maar", "of", "want", "dus", (const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
 /*}}}*/
 
+/**
+ * Test if the word is a nominalization.  This function uses
+ * docLanguage to determine the used language.
+ */
+ 
 static int nominalization(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Nominalisierungsendungen */ /*{{{*/
@@ -400,40 +430,80 @@ static int nominalization(const char *word, size_t l) /*{{{*/
      "tion", "ment", "ence", "ance", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch nominalization suffixes (verzelfstandigde werkwoorden */ /*{{{*/
+  {
+     /* vereenvoudigd */
+     "tie", "heid", "ing", "end", "ende", (const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   /* exclude words too short to have such long suffixes */
   if (l < 7) return 0;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list) if (wordcmp(*list,word+l-strlen(*list))==0) return 1; else ++list;
   return 0;
 }
 /*}}}*/
 
+/**
+ * Test if the word is an sub conjunction.  This function uses
+ * docLanguage to determine the used language.
+ */
+ 
 static int subConjunction(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* unterordnende Konjunktionen */ /*{{{*/
   {
-    "weil", "da", "dadurch", "wenn", "falls", "sofern", "obwohl",
-    "obgleich", "als", "nachdem", "whrend", "wie", "ob", "je",
-    "desto", "damit", "dass", "indem", "um zu", (const char*)0
+    /* bei Nebenstzen */
+    "als", "als dass", "als da", "als ob", "anstatt dass", "anstatt da",
+    "ausser dass", "ausser da", "ausser wenn", "bevor", "bis", "da", "damit",
+    "dass", "da", "ehe", "falls", "indem", "je", "nachdem", "ob", "obgleich",
+    "obschon", "obwohl", "ohne dass", "ohne da", "seit", "so da", "sodass",
+    "sobald", "sofern", "solange", "so oft", "statt dass", "statt da",
+    "whrend", "weil", "wenn", "wenn auch", "wenngleich", "wie", "wie wenn",
+    "wiewohl", "wobei", "wohingegen", "zumal"
+    /* bei Infinitivgruppen */
+    "als zu", "anstatt zu", "ausser zu", "ohne zu", "statt zu", "um zu",
+    (const char*)0
   };
   /*}}}*/
   static const char *en[]= /* subordinating conjunctions */ /*{{{*/
   {
-    "after","because", "lest", "till", "'til", "although", "before", 
+    "after", "because", "lest", "till", "'til", "although", "before", 
     "now that", "unless", "as", "even if", "provided that", "provided", 
     "until", "as if", "even though", "since", "as long as", "so that",
     "whenever", "as much as", "if", "than", "as soon as", "inasmuch",
     "in order that", "though", "while", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch subordinating conjunctions (onderschikeknde voegwoorden */ /*{{{*/
+  {
+	/* onderschikkende voegwoorden */
+    "aangezien", "als", "alsof", "behalve", "daar", "daarom", "dat", 
+	"derhalve", "doch", "doordat", "hoewel", "mits", "nadat", 
+    "noch", "ofschoon", "omdat", "ondanks", "opdat", "sedert", "sinds",
+	"tenzij", "terwijl", "toen", "totdat", "voordat", "wanneer",
+	"zoals", "zodat", "zodra", "zonder dat", 
+	
+	/* infitief constructies */
+	"om te", (const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list)
   {
     if (wordcmp(*list,word)==0) 
@@ -447,6 +517,11 @@ static int subConjunction(const char *word, size_t l) /*{{{*/
 }
 /*}}}*/
 
+/**
+ * Test if the word is an preposition.  This function uses
+ * docLanguage to determine the used language.
+ */
+ 
 static int preposition(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Prpositionen */ /*{{{*/
@@ -478,10 +553,25 @@ static int preposition(const char *word, size_t l) /*{{{*/
     "by", "of", "in", "to", "near", "of", "from",  (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch prepositions (voorzetsels) */ /*{{{*/
+  {
+    "", "aan", "ad", "achter", "behalve", "beneden", "betreffende", "bij", "binnen", "blijkens", "boven", "buiten", 
+	"circa", "conform", "contra", "cum", "dankzij", "door", "gedurende", "gezien", "hangende", "in", "ingevolge", 
+	"inzake", "jegens", "krachtens", "langs", "met", "middels", "mits", "na", "naar", "naast", "nabij", "namens", 
+	"niettegenstaande", "nopens", "om", "omstreeks", "omtrent", "ondanks", "onder", "ongeacht", "onverminderd", 
+	"op", "over", "overeenkomstig", "per", "plus", "richting", "rond", "rondom", "sedert", "staande", "te", "tegen", 
+	"tegenover", "ten", "ter", "tijdens", "tot", "tussen", "uit", "uitgezonderd", "van", "vanaf", "vanuit", "vanwege", 
+	"versus", "via", "volgens", "voor", "voorbij", "wegens", "zonder",  (const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list)
   {
     if (wordcmp(*list,word)==0)
@@ -495,6 +585,11 @@ static int preposition(const char *word, size_t l) /*{{{*/
 }
 /*}}}*/
 
+/**
+ * Test if the word is an auxiliary verb.  This function uses
+ * docLanguage to determine the used language.
+ */
+ 
 static int auxVerb(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Hilfsverben */ /*{{{*/
@@ -505,7 +600,7 @@ static int auxVerb(const char *word, size_t l) /*{{{*/
     "wrdest", "wrden", "wrdet",
     "knnen", "kann", "kannst", "knnt", "konnte", "konntest", "konnten",
     "konntet", "gekonnt", "knnte", "knntest", "knnten", "knntet",
-    "mssen", "muss", "musst", "msst", "musste", "musstest", "mussten",
+    "mssen", "muss", "mu", "musst", "msst", "musste", "musstest", "mussten",
     "gemusst", "msste", "msstest", "mssten", "msstet",
     "sollen", "soll", "sollst", "sollt", "sollte", "solltest", "solltet",
     "sollten", "gesollt",
@@ -519,10 +614,32 @@ static int auxVerb(const char *word, size_t l) /*{{{*/
     (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch auxiliary verbs (hulpwerkwoorden) */ /*{{{*/
+  {
+	/* in combinatie met voltooid deelwoord */
+	/* hebben */	"heb",  "hebt", "heeft", "hebben", "had",    "hadden",  "gehad", 
+	/* zijn */		/* "ben",  "bent", "is",    "zijn",   "was",    "waren",   "geweest", */
+	/* worden */	"word", "wordt",         "worden", "werd",   "werden",  "geworden",
+
+	/* in combinatie met infinitief */
+	/* kunnen */	"kan", "kan",            "kunnen", "kon",    "konden",  "gekund",
+	/*  willen */	"wil",                   "willen", "wilde",  "wilden",  "gewild", "wou", "wouden", 
+	/* zullen */		"zal", "zult",           "zullen", "zou",    "zouden", 
+	/* mogen */	"mag",                   "mogen",  "mocht",  "mochten", "gemogen",
+	/*moeten */	"moet",                  "moeten", "moest",  "moesten", "gemoeten",
+	/* hoeven  */	"hoef", "hoeft",         "hoeven", "hoefde", "hoefden", "gehoeven", 
+	/* doen */		"doe", "doet",           "doen",   "deed",   "deden",   "gedaan",
+	(const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list)
   {
     if (wordcmp(*list,word)==0) 
@@ -536,6 +653,11 @@ static int auxVerb(const char *word, size_t l) /*{{{*/
 }
 /*}}}*/
 
+/**
+ * Test if the word is an 'to be' verb.  This function uses
+ * docLanguage to determine the used language.
+ */
+ 
 static int tobeVerb(const char *word, size_t l) /*{{{*/
 {
   static const char *de[]= /* Hilfsverb sein */ /*{{{*/
@@ -550,10 +672,20 @@ static int tobeVerb(const char *word, size_t l) /*{{{*/
      "be", "being", "was", "were", "been", "are", "is", (const char*)0
   };
   /*}}}*/
+  static const char *nl[]= /* Dutch auxiliary verb to be (zijn) */ /*{{{*/
+  {
+  "ben", "bent", "is", "zijn", "was", "waren", "geweest",
+  (const char*)0
+  };
+  /*}}}*/
+
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
-  else list=en;
+  else if (strncmp(docLanguage,"en",2)==0) list=en;
+  else if (strncmp(docLanguage,"nl",2)==0) list=nl;
+  else assert(0);
+
   while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
@@ -565,10 +697,13 @@ static int tobeVerb(const char *word, size_t l) /*{{{*/
  */
 static int vowel(char c) /*{{{*/
 {
+  if (c=='y' && strncmp(docLanguage,"en",2)) return 1;
+
   switch (lc_ctype_int)
   {
-    case ASCII: return (c=='a' || c=='e' || c=='i' || c=='o' || c=='u' || c=='y');
-    case ISO_8859_1: return (c=='a' || c=='' || c=='e' || c=='i' || c=='o' || c=='' || c=='u' || c=='' || c=='y');
+    case ASCII: return (c=='a' || c=='e' || c=='i' || c=='o' || c=='u'|| c=='y');		/* JDL */
+    case ISO_8859_1: return (c=='a' || c=='' || c=='e' || c=='i' || c=='o' || c=='' || c=='u' || c=='' ||		/* JDL */
+							 c=='' || c=='' || c=='' || c=='' || c=='i' || c=='' || c=='y');
     default: assert(0);
   }
 }
@@ -623,6 +758,36 @@ static int syll_de(const char *s, size_t l) /*{{{*/
 }
 /*}}}*/
 
+/**
+ * Count syllables for Dutch words by counting vowel-consonant or
+ * consonant-vowel pairs, depending on the first character being a vowel or
+ * not.  If it is, a trailing e will be handled with a special rule.  This
+ * algorithm fails on "vor-ueber".
+ * @param s the word
+ * @param l the word's length
+ */
+static int syll_nl(const char *s, size_t l) /*{{{*/
+{  
+  int count=0;
+  size_t ol=l;
+ 
+  if (vowel(*s))  
+  while (l) 
+  {
+    if (l>=2 && vowel(*s) && !vowel(*(s+1))) { ++count; s+=2; l-=2; }
+    else if (l==1 && ol>1 && !vowel(*(s-1)) && *s=='e') { ++count; s+=1; l-=1; }
+    else { ++s; --l; }
+  }
+  else
+  while (l)
+  {
+    if (l>=2 && !vowel(*s) && vowel(*(s+1))) { ++count; s+=2; l-=2; }
+    else { ++s; --l; }
+  }
+  return (count==0 ? 1 : count);
+}
+/*}}}*/
+
 /**
  * Count syllables.  First, charset is set to the used character set.
  * Depending on the language, the right counting function is called.
@@ -633,7 +798,10 @@ static int syll(const char *s, size_t l) /*{{{*/
 {
   assert(s!=(const char*)0);
   assert(l>=1);
+  
   if (strncmp(docLanguage,"de",2)==0) return syll_de(s,l);
+  else if (strncmp(docLanguage,"en",2)==0) return syll_en(s,l);
+  else if (strncmp(docLanguage,"nl",2)==0) return syll_nl(s,l);
   else return syll_en(s,l);
 }
 /*}}}*/
@@ -689,17 +857,17 @@ static void style(const char *str, size_t length, const char *file, int line) /*
   int count;
   int passive=0;
   int nom=0;
-  const char *s=str;
+  const char *s=str,*end=s+length;
 
   if (length==0) { ++paragraphs; return; }
   assert(str!=(const char*)0);
   assert(length>=2);
   phraseEnd = (const char*)0;
-  while (*s)
+  while (s<end)
   {
     if (inword)
     {
-      if (!isalpha(*s))
+      if (!isalpha(*s) && *s!='-' && !endingInPossesiveS(str,s-str+2))
       {
         inword=0;
         count=syll(s-wordLength,wordLength);
@@ -759,17 +927,17 @@ static void style(const char *str, size_t length, const char *file, int line) /*
     }
     else if (innumber)
     {
-      if (!isdigit(*s))
-      {
-        innumber=0;
-        ++syllables;
-      }
-      else
+      if (isdigit(*s) || ((*s=='.' || *s==',') && isdigit(*(s+1))))
       {
         ++wordLength;
         ++characters;
         ++sentLetters;
       }
+      else
+      {
+        innumber=0;
+        ++syllables;
+      }
     }
     else
     {
@@ -815,6 +983,16 @@ static void style(const char *str, size_t length, const char *file, int line) /*
 }
 /*}}}*/
 
+static void print_usage(FILE *handle) /*{{{*/
+{
+  fputs(_("\
+Usage: style [-L language] [-l length] [-r ari] [file ...]\n\
+       style [--language language] [--print-long length] [--print-ari ari]\n\
+             [file ...]\n\
+       style --version\n"),handle);
+}
+/*}}}*/
+
 int main(int argc, char *argv[]) /*{{{*/
 {
   /* variables */ /*{{{*/
@@ -835,7 +1013,10 @@ int main(int argc, char *argv[]) /*{{{*/
 
   /* locale */ /*{{{*/
   setlocale(LC_ALL,"");
-  catd=catopen("style",0);
+#ifdef HAVE_GETTEXT
+  bindtextdomain("diction", LOCALEDIR);
+  textdomain("diction");
+#endif
   /*}}}*/
   /* parse options */ /*{{{*/
 #if 0
@@ -860,6 +1041,11 @@ int main(int argc, char *argv[]) /*{{{*/
     case 'L':
     {
       docLanguage=optarg;
+      if (strncmp(docLanguage,"de",2) && strncmp(docLanguage,"en",2) && strncmp(docLanguage,"nl",2))
+      {
+        fprintf(stderr,_("style: Incorrect language option `%s'.\n"),docLanguage);
+        exit(1);
+      }
       break;
     }
     case 'r':
@@ -891,91 +1077,92 @@ int main(int argc, char *argv[]) /*{{{*/
   }
   if (usage==1)
   {
-    fputs(USAGE1,stderr);
-    fputs(USAGE2,stderr);
-    fputs(USAGE3,stderr);
+    print_usage(stderr);
     fputs("\n",stderr);
-    fputs(MORE,stderr);
+    fputs(_("Try style -h|--help for more information.\n"),stderr);
     exit(1);
   }
   else if (usage==2)
   {
-    fputs(USAGE1,stdout);
-    fputs(USAGE2,stdout);
-    fputs(USAGE3,stdout);
+    print_usage(stdout);
     fputs("\n",stdout);
-    fputs(HELP1,stdout);
+    fputs(_("Analyse surface characteristics of a document.\n\n"),stdout);
+    fputs(_("\
+-L, --language          set the document language.\n\
+-l, --print-long        print all sentences longer than <length> words\n\
+-r, --print-ari         print all sentences with an ARI greater than than <ari>\n\
+-p, --print-passive     print all sentences phrased in the passive voice\n\
+-N, --print-nom         print all sentences containing nominalizations\n\
+-n, --print-nom-passive print all sentences phrased in the passive voice or\n\
+                        containing nominalizations\n"),stdout);
+    fputs(_("\
+-h, --help              print this message\n\
+    --version           print the version\n"),stdout);
     fputs("\n",stdout);
-    fputs(HELP2,stdout);
-    fputs(HELP3,stdout);
-    fputs(HELP4,stdout);
-    fputs(HELP5,stdout);
-    fputs(HELP6,stdout);
-    fputs(HELP7,stdout);
-    fputs(HELP8,stdout);
-    fputs(HELP9,stdout);
-    fputs("\n",stdout);
-    fputs(BUGS,stdout);
+    fputs(_("Report bugs to <michael@moria.de>.\n"),stdout);
     exit(0);
   }
   /*}}}*/
   newHit(&lengths);
-  if (optind==argc) sentence("style",catd,stdin,"(stdin)",style,docLanguage);
+  if (optind==argc) sentence("style",stdin,"(stdin)",style,docLanguage);
   else while (optind<argc)
   {
     FILE *fp;
-    if ((fp=fopen(argv[optind],"r"))==(FILE*)0) fprintf(stderr,OPENFAILED,argv[optind],strerror(errno));
+    if ((fp=fopen(argv[optind],"r"))==(FILE*)0) 
+      fprintf(stderr,_("style: Opening `%s' failed (%s).\n"),argv[optind],strerror(errno));
     else
     {
-      sentence("style",catd,fp,argv[optind],style,docLanguage);
+      sentence("style",fp,argv[optind],style,docLanguage);
       fclose(fp);
     }
     ++optind;
   }
   if (sentences==0)
   {
-    printf(NOSENT);
+    printf(_("No sentences found.\n"));
   }
   else
   {
+    double fl;
     int wsg;
     int lixg;
     int i,shortLength,shortSent,longLength,longSent;
 
-    printf(GRADES);
-    printf("        %s: %.1f\n",KINCAID,kincaid(syllables,words,sentences));
-    printf("        %s: %.1f\n",ARI,ari(characters,words,sentences));
-    printf("        %s: %.1f\n",CL,coleman_liau(characters,words,sentences));
-    printf("        %s: %.1f\n",FLESCH,flesch(syllables,words,sentences));
-    printf("        %s: %.1f\n",FOG,fog(words,bigwords,sentences));
+    printf(_("readability grades:\n"));
+    printf("        %s: %.1f\n","Kincaid",kincaid(syllables,words,sentences));
+    printf("        %s: %.1f\n","ARI",ari(characters,words,sentences));
+    printf("        %s: %.1f\n","Coleman-Liau",coleman_liau(characters,words,sentences));
+    fl=flesch(syllables,words,sentences);
+    printf("        %s: %.1f%s\n","Flesch Index",fl,fl>=60 && fl<=70 ? _("/100 (plain English)") : _("/100"));
+    printf("        %s: %.1f\n","Fog Index",fog(words,bigwords,sentences));
 #ifdef MIGHT_BE_WRONG
-    printf("        %s: %.1f\n",WSFT,wstf(words,shortwords,longwords,bigwords,sentences));
-    printf("        %s: %.1f = ",WSI,wheeler_smith(&wsg,words,bigwords,sentences));
-    if (wsg==0) printf(BELOW5);
-    else if (wsg==99) printf(AHEAD10);
-    else printf(YEAR,wsg);
+    printf("        %s: %.1f\n","1. WSTF Index",wstf(words,shortwords,longwords,bigwords,sentences));
+    printf("        %s: %.1f = ","Wheeler-Smith Index",wheeler_smith(&wsg,words,bigwords,sentences));
+    if (wsg==0) printf(_("below school year 5\n"));
+    else if (wsg==99) printf(_("higher than school year 10\n"));
+    else printf(_("school year %d\n"),wsg);
 #endif
-    printf("        %s: %.1f = ",LIX,lix(&lixg,words,longwords,sentences));
-    if (lixg==0) printf(BELOW5);
-    else if (lixg==99) printf(AHEAD11);
-    else printf(YEAR,lixg);
-    printf("        %s: %.1f\n",SMOG,smog(bigwords,sentences));
+    printf("        %s: %.1f = ",_("Lix"),lix(&lixg,words,longwords,sentences));
+    if (lixg==0) printf(_("below school year 5\n"));
+    else if (lixg==99) printf(_("higher than school year 11\n"));
+    else printf(_("school year %d\n"),lixg);
+    printf("        %s: %.1f\n",_("SMOG-Grading"),smog(bigwords,sentences));
 
-    printf(SENTINFO);
-    printf(CHARS,characters);
-    printf("        %d words, average length %.2f characters = %.2f syllables\n",words,((double)characters)/words,((double)syllables)/words);
-    printf("        %d sentences, average length %.1f words\n",sentences,((double)words)/sentences);
+    printf(_("sentence info:\n"));
+    printf(_("        %d characters\n"),characters);
+    printf(_("        %d words, average length %.2f characters = %.2f syllables\n"),words,((double)characters)/words,((double)syllables)/words);
+    printf(_("        %d sentences, average length %.1f words\n"),sentences,((double)words)/sentences);
     shortLength=((double)words)/sentences-4.5;
     if (shortLength<1) shortLength=1;
     for (i=0,shortSent=0; i<=shortLength; ++i) shortSent+=lengths.data[i];
-    printf(SHORTSENT,100*shortSent/sentences,shortSent,shortLength);
+    printf(_("        %d%% (%d) short sentences (at most %d words)\n"),100*shortSent/sentences,shortSent,shortLength);
     longLength=((double)words)/sentences+10.5;
     for (i=longLength,longSent=0; i<=lengths.size; ++i) longSent+=lengths.data[i];
-    printf("        %d%% (%d) long sentences (at least %d words)\n",100*longSent/sentences,longSent,longLength);
-    printf("        %d paragraphs, average length %.1f sentences\n",paragraphs,((double)sentences)/paragraphs);
-    printf("        %d%% (%d) questions\n",100*questions/sentences,questions);
-    printf("        %d%% (%d) passive sentences\n",100*passiveSent/sentences,passiveSent);
-    printf("        longest sent %d wds at sent %d; shortest sent %d wds at sent %d\n",longestLength,longestLine,shortestLength,shortestLine);
+    printf(_("        %d%% (%d) long sentences (at least %d words)\n"),100*longSent/sentences,longSent,longLength);
+    printf(_("        %d paragraphs, average length %.1f sentences\n"),paragraphs,((double)sentences)/paragraphs);
+    printf(_("        %d%% (%d) questions\n"),100*questions/sentences,questions);
+    printf(_("        %d%% (%d) passive sentences\n"),100*passiveSent/sentences,passiveSent);
+    printf(_("        longest sent %d wds at sent %d; shortest sent %d wds at sent %d\n"),longestLength,longestLine,shortestLength,shortestLine);
 
 /*
 Missing output:
@@ -992,27 +1179,21 @@ word usage:
         noun 25.0% (1) adj 25.0% (1) pron 25.0% (1)
         nominalizations   0 % (0)
 */
-        if (strncmp(docLanguage,"en",2)==0)
-        {
-          printf("word usage:\n");
-          printf("        verb types:\n");
-          printf("        to be (%d) auxiliary (%d) \n", tobeVerbs, auxVerbs);
-          printf("        types as %% of total:\n");
-          printf("        conjunctions %1.f% (%d) pronouns %1.f% (%d) prepositions %1.f% (%d)\n", 
-                  (100.0*(conjunctions+subConjunctions))/words, 
-                  conjunctions+subConjunctions, 
-                  (100.0*pronouns)/words, pronouns, (100.0*prepositions)/words,
-                  prepositions);
-          printf("        nominalizations %1.f% (%d)\n",
-                  (100.0*nominalizations)/words, nominalizations);
-        }
+        printf(_("word usage:\n"));
+        printf(_("        verb types:\n"));
+        printf(_("        to be (%d) auxiliary (%d) \n"), tobeVerbs, auxVerbs);
+        printf(_("        types as %% of total:\n"));
+        printf(_("        conjunctions %1.f% (%d) pronouns %1.f% (%d) prepositions %1.f% (%d)\n"), 
+                (100.0*(conjunctions+subConjunctions))/words, 
+                conjunctions+subConjunctions, 
+                (100.0*pronouns)/words, pronouns, (100.0*prepositions)/words,
+                prepositions);
+        printf(_("        nominalizations %1.f% (%d)\n"),
+                (100.0*nominalizations)/words, nominalizations);
 
-        printf("sentence beginnings:\n");
-        printf("        pronoun (%d) interrogative pronoun (%d) article (%d)\n",beginPronouns,beginInterrogativePronouns,beginArticles);
-        if (strncmp(docLanguage,"en",2)==0)
-        {
-          printf("        subordinating conjunction (%d) conjunction (%d) preposition (%d)\n", beginSubConjunctions,beginConjunctions,beginPrepositions);
-        }
+        printf(_("sentence beginnings:\n"));
+        printf(_("        pronoun (%d) interrogative pronoun (%d) article (%d)\n"),beginPronouns,beginInterrogativePronouns,beginArticles);
+        printf(_("        subordinating conjunction (%d) conjunction (%d) preposition (%d)\n"), beginSubConjunctions,beginConjunctions,beginPrepositions);
 
 /*
         subject opener: noun (0) pron (1) pos (0) adj (0) art (0) tot 100%
